#!/usr/bin/perl
### SysChk 3.1 
### By: h0e  <h0e@IHackedThisBox.com>
### Http://www.IHackedThisBox.com
### - m0o

## Description ##
## SysChk is a tool to aide in monitoring of files & directorys for changes.
## SysChk will inventory owners, groups, permissions, modification times, as
## well as md5 hash computations of a file. In the event of a change in any 
## file or directory monitored by SysChk you will be notified.


use DB_File;
use Getopt::Std;
use File::Find;
use vars qw(%opt %DB);
getopts('hicdf:s:l:', \%opt);
$version = "SysChk 3.1";
$cfg_file = "/usr/local/etc/syschk.conf";
$dbs_file = "/var/lib/syschk.sk";
$CHANGE; $LOGGING;
$nfiles=0; $fchanges=0; $dfiles=0; $tchanges=0; $hchanges=0; $mchanges=0; $filecount=0;
my @DIR;


tie %DB, 'DB_File', $dbs_file ||
    die "Unable to tie $dbs_file: $!\n";


my $cfg_file = "$opt{f}" || $cfg_file;
my $dbs_file = "$opt{s}" || $dbs_file;


open(CONFIG, "$cfg_file") ||
    die "Unable to open CFG: ($!)\n";

while (<CONFIG>) {
chomp;
if (/^DIR=/) { 
  ($key, $value) = split(/=/, $_); 
  push (@DIR, $value);  }
}

# debugging
if ($opt{d}) { foreach (sort keys %DB) { print "$_ -> $DB{$_}\n"; } }
#debugging end


print "\n\t\t\t\t $version\n";

if ($opt{i}) {
print "\t\t-- [Starting]  Database Initialization   --\n\n";
find(\&proc_files, @DIR);
foreach (keys %DB) { $filecount++; }
 print "
 \t\t\tSummary:
 \t\t\tFiles In Database: $filecount
 \t\t\tNew Files: $nfiles
 \n";
print "\t\t-- [Completed] Database Initialization   --\n\n";
}

if ($opt{c}) {
print "\t\t--  [Starting] File System Integrity Check   --\n";
find(\&proc_files, @DIR);
print "\t\t\t   No Changes Detected.\n\n" if (!$CHANGE);
print "
 \t\t\tSummary:
 \t\t\tFiles In Database: $filecount
 \t\t\tHash Changes  $hchanges
 \t\t\tMode Changes  $mchanges
 \t\t\tTime Changes: $tchanges
 \t\t\tNew Files: $nfiles
 \t\t\tDeleted Files: $dfiles
\n\n";
print "\t\t-- [Completed] File System Integrity Check   --\n\n";
foreach (keys %DB) { $filecount++; }
 }

if ($opt{l}) {  $LOGGING=1; }
if ($opt{h}) {  usage();    }


sub init {
$nfiles++;
   #unlink "$dbfile";
   $info = shift;
  ($fname, $sum, $mode, $uid, $gid, $mtime) = split(/:/, $info);
   $DB{$fname} = "$sum:$mode:$uid:$gid:$mtime";
}




sub proc_files {
$fname = $File::Find::name;

next if (-d $_);
 if (open(FILE, $_)) {
  my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, 
      $atime, $mtime, $ctime, $blksize, $blocks) = stat(FILE);
   
      $fhash = `md5sum $_`;
     ($sum, $filename) = split(/  /, $fhash);
      $info = "$fname:$sum:$mode:$uid:$gid:$mtime"; 
      init($info) if $opt{i};
      check_files($info) if $opt{c};  }  
}

sub check_files { 
    $info = shift;
    $fname = $File::Find::name;
   ($fname, $digest, $mode, $uid, $gid, $mtime) = split(/:/, $info);

 my $finfo = $DB{$fname} || 0;
    $DB{$fname} = $finfo; 

if ($finfo) { my ($odigest,$omode,$ouid,$ogid,$omtime) = split (':', $finfo);

unless ($digest eq $odigest) { $CHANGE = 1;
        $fchanges++;
        logging("Hash Changed: $fname\n Original: $odigest \n New: $digest\n"); }        

unless ($mode eq $omode) {     $CHANGE = 1;
        $mchanges++;
        logging("Mode Changed: $fname\n Original: $omode \n New: $mode\n");   }        

unless ($uid eq $ouid) {       $CHANGE = 1;
        $fchanges++;
        logging("User ID Changed: $fname\n Original: $ouid \n New: $uid\n");  }        

unless ($gid eq $ogid) {       $CHANGE = 1;
        $fchanges++;
        logging("Group ID Changed: $fname\n Original: $ogid \n New: $gid\n"); }        

unless ($mtime eq $omtime) {   $CHANGE = 1;
        $mtime = scalar(localtime($mtime));
        $omtime = scalar(localtime($omtime));
        $tchanges++;
        logging("\nModification time has changed:  \n Original: $omtime \n New: $mtime\n File: $fname\n"); }
} else { 
       $nfiles++;
       logging("\nNew File: $fname\n");
       print "Would you like to add this file? [y/N] ";
       chomp($in = <STDIN>);
     if ($in eq "y") { $DB{$fname} = "$digest:$mode:$uid:$gid:$mtime"; }
        } close(FILE);   
 }

foreach (keys %DB) { 
     if (!-e $_) {            $CHANGE=1;
         $dfiles++;
         logging("Can not open $_: ($!)\n"); 
         print "Would you like to update this change? [y/N]: ";
         chomp($in = <STDIN>);
     if ($in eq "y") { delete $DB{$_} } 
         } 
 }  untie %DB;



sub logging {  
$log_entry = shift;
if (!$opt{l}) { die "No logfile specified\n"; }

if 
 (!$LOGGING) { print "$log_entry\n"; }
 
elsif 
    ($LOGGING = 1) { 
 open (LOGFILE, ">>$opt{l}") ||
   die "Unable to open $opt{l}: ($!)\n";
       print LOGFILE "$log_entry\n";   } 
       close(LOGFILE);
 }


sub usage {
 print <<EOF;
   
  \t\t\tBy: h0e
  \t\t\tHttp://www.IHackedThisBox.com/

Description:
 SysChk is a tool to assit in file integrity monitoring.
 This tool will monitor the following:
 UID, GID, Mode, MD5 Hash, Modification Time.
See Also: CHANGES, TODO

Options:
 -h This menu.
 -f <cfg file>.
 -s <database file>.
 -i Initialize database.
 -c Check filesystem against database.
 -d Dump the whole database.
\nQuestions/Comments? 	h0e\@IHackedThisBox.com
EOF
}
